@@ -19,7 +19,7 @@ Start a service task (i.e. something you depend on) with
 ``asyncscope.spawn_service``. Start a subtask which should run in parallel
 and which you're going to wait on with ``asyncscope.spawn``.
 
-Scope objects are handled in the background, though you are free to 
+Scope objects are handled in the background, though you are free to
 create one.
 
 """
@@ -29,7 +29,7 @@ import anyio
 import anyio.abc
 from contextvars import ContextVar
 from contextlib import asynccontextmanager
-from typing import Any
+from typing import Any, Set, Dict
 
 __all__ = ["run"]
 
@@ -131,11 +131,22 @@ class Scope:
 
         Never use this method to check whether you need to call `service`.
         """
-        s = self._set[name]
+        s = self.lookup_scope(name)
         if not s._data_lock.is_set():
             raise KeyError(name)
         return s._data
 
+    def lookup_scope(self, name):
+        """
+        Return the scope associated with some name.
+
+        Raises KeyError if the named scope doesn't exist.
+
+        Use this if you need to add a dependency.
+        """
+        s = self._set[name]
+        return s
+
     async def register(self, data: Any):
         """
         Register some data with this scope.
@@ -171,12 +182,13 @@ class Scope:
                     async with anyio.open_cancel_scope(shield=True):
                         await self.cancel_immediate()
         finally:
-            scope.reset(self._scope)
-            await self._data_lock.set()
-            async with anyio.open_cancel_scope(shield=True):
-                await self._done.set()
-                for p in list(self._prev):
-                    await self.release(p)
+            if self._scope is not None:  # error in setup
+                scope.reset(self._scope)
+                await self._data_lock.set()
+                async with anyio.open_cancel_scope(shield=True):
+                    await self._done.set()
+                    for p in list(self._prev):
+                        await self.release(p)
             self._tg = None
             self._scope = None
 
@@ -224,7 +236,6 @@ class Scope:
 
         Ordered top-down.
         """
-        todo = set(self._next)
         seen = set()
 
         def deps(s):
@@ -283,7 +294,7 @@ class Scope:
     async def cancel(self):
         """
         Cancel this scope.
-        
+
         Do not call directly!
         """
         if self._tg:
@@ -366,7 +377,7 @@ class ScopeSet:
         """
         Context manager for a new scope set
         """
-        s = Scope(self, "_main")
+        s = Scope(self, "_main", new=True)
         async with anyio.create_task_group() as tg:
             self._tg = tg
             async with s._ctx():
@@ -385,7 +396,7 @@ class ScopeSet:
         return await self._ctx_.__aenter__()
 
     async def __aexit__(self, *tb):
-        return await self._ctx_.__aexit__(*tb)
+        return await self._ctx_.__aexit__(*tb)  # pylint:disable=no-member  # YES IT HAS
 
 
 async def spawn_service(proc, *args, **kwargs):
@@ -403,7 +414,7 @@ async def spawn_service(proc, *args, **kwargs):
 
 async def spawn(proc, *args, **kwargs):
     """
-    Run 'proc' as a subtask in the current scope.
+    Run 'proc' as a subtask of the current scope.
 
     Returns: a cancel scope, useable to cancel this subtask.
     """
@@ -427,10 +438,19 @@ def lookup(name):
     Raises KeyError if the named scope doesn't exist or has not
     provided any data.
 
-    Use this if you need a temporary reference to a scope which might
-    depend on the current one.
+    Use this if you need a temporary reference to the data of a scope which
+    might depend on the current one.
+    """
+    return scope.get().lookup(name)
 
-    Never use this method to check whether you need to call `service`.
+
+def lookup_scope(name):
+    """
+    Return the scope associated with some name.
+
+    Raises KeyError if the named scope doesn't exist.
+
+    Use this if you need to add a dependency.
     """
     return scope.get().lookup(name)
 
@@ -457,22 +477,16 @@ async def no_more_dependents():
 
 
 @asynccontextmanager
-async def main_scope(_name_="main"):
+async def main_scope(name="_main"):
     """
     This context manager provides you with a new "main" scope, i.e. one you
     can start service tasks in.
     """
-    async with anyio.create_task_group() as tg, Scope(tg, _name_, new=True)._ctx() as s:
+
+    async with anyio.create_task_group() as tg, ScopeSet() as s:
         try:
             yield s
         finally:
-            await s.cancel_dependents()
+            await s.cancel_dependents()  # should not be any but …
+            await s.cancel()
 
-
-async def _main(proc, args, kwargs):
-    async with main_scope():
-        return await proc(*args, **kwargs)
-
-
-def run(proc, *args, **kwargs):
-    return anyio.run(_main, proc, args, kwargs, backend="trio")
